package mi.hdm.recipes;

import mi.hdm.exceptions.InvalidCategoryException;
import org.junit.jupiter.api.Assertions;
import org.junit.jupiter.api.Test;

public class CategoryTest {
    CategoryManager underTest = CategoryManager.getInstance();

    @Test
    public void testEqualCategories() {
        underTest.clearCategories();
        Category c1 = new Category("Abend", 0x00FF00);
        Category c2 = new Category("LEcker", 0x13);
        Category c3 = new Category("ew", 0x13);
        Category c4 = new Category("ew", 0x100);

        Assertions.assertNotEquals(c1, c2);
        Assertions.assertEquals(c3, c4);
        Assertions.assertEquals(c2, c3);
    }

    @Test
    public void shouldNotBeAbleToAddIdenticalName() {
        underTest.clearCategories();
        underTest.addCategory("Mittag", 0x100000);
        underTest.addCategory("Mittag", 0x100040);

        Assertions.assertEquals(1, underTest.getAllCategories().size());
    }

    @Test
    public void shouldNotBeAbleToAddIdenticalCode() {
        underTest.clearCategories();
        underTest.addCategory("Morgen", 0x100040);
        underTest.addCategory("Mittag", 0x100040);

        Assertions.assertEquals(1, underTest.getAllCategories().size());
    }

    @Test
    public void shouldBeAbleToAddCategory() {
        underTest.clearCategories();
        underTest.addCategory("Morgen", 0x100000);
        underTest.addCategory("Abend", 0x100010);

        Assertions.assertEquals(2, underTest.getAllCategories().size());
    }

    @Test
    public void canRemoveCategoryByName() {
        //given
        underTest.clearCategories();
        underTest.addCategory("Jan ist cool", 0xFF0000);

        //when
        underTest.deleteCategory("Jan ist cool");

        //expect
        Assertions.assertEquals(0, underTest.getAllCategories().size());
    }

    @Test
    public void canNotAddCategoryWithEmptyName() {
        Assertions.assertThrows(InvalidCategoryException.class, () -> new Category("", 0x1000));
    }

    @Test
    public void canNotAddCategoryWithNullName() {
        Assertions.assertThrows(InvalidCategoryException.class, () -> new Category(null, 0x1000));
    }

    @Test
    public void canNotAddCategoryWithNegativeCode() {
        Assertions.assertThrows(InvalidCategoryException.class, () -> new Category("Negative", -100));
    }

    @Test
    public void canNotAddCategoryWithCodeTooHigh() {
        Assertions.assertThrows(InvalidCategoryException.class, () -> new Category("too high", Integer.MAX_VALUE));
    }
}
